"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ApolloServerPluginCacheControlDisabled = exports.ApolloServerPluginCacheControl = exports.PolicyUpdater = void 0;
const apollo_server_types_1 = require("apollo-server-types");
const graphql_1 = require("graphql");
class PolicyUpdater {
    constructor(requestContext) {
        this.requestContext = requestContext;
        this.overallCachePolicyIsUncacheable = false;
    }
    addHint(hint) {
        var _a;
        if (this.overallCachePolicyIsUncacheable) {
            return;
        }
        if (hint.maxAge === 0) {
            this.requestContext.overallCachePolicy = undefined;
            this.overallCachePolicyIsUncacheable = true;
            return;
        }
        if (!this.requestContext.overallCachePolicy) {
            if (hint.maxAge === undefined) {
                throw Error("Shouldn't happen: first hint has undefined maxAge?");
            }
            this.requestContext.overallCachePolicy = {
                maxAge: hint.maxAge,
                scope: (_a = hint.scope) !== null && _a !== void 0 ? _a : apollo_server_types_1.CacheScope.Public,
            };
            return;
        }
        if (hint.maxAge !== undefined &&
            hint.maxAge < this.requestContext.overallCachePolicy.maxAge) {
            this.requestContext.overallCachePolicy.maxAge = hint.maxAge;
        }
        if (hint.scope === apollo_server_types_1.CacheScope.Private) {
            this.requestContext.overallCachePolicy.scope = apollo_server_types_1.CacheScope.Private;
        }
    }
}
exports.PolicyUpdater = PolicyUpdater;
function ApolloServerPluginCacheControl(options = Object.create(null)) {
    return {
        __internal_plugin_id__() {
            return 'CacheControl';
        },
        requestDidStart(requestContext) {
            var _a, _b;
            const defaultMaxAge = (_a = options.defaultMaxAge) !== null && _a !== void 0 ? _a : 0;
            const calculateHttpHeaders = (_b = options.calculateHttpHeaders) !== null && _b !== void 0 ? _b : true;
            const { __testing__cacheHints } = options;
            return {
                executionDidStart: () => {
                    let policyUpdater;
                    if (!requestContext.overallCachePolicy) {
                        policyUpdater = new PolicyUpdater(requestContext);
                    }
                    return {
                        willResolveField({ info }) {
                            let hint = {};
                            const targetType = graphql_1.getNamedType(info.returnType);
                            if (targetType instanceof graphql_1.GraphQLObjectType ||
                                targetType instanceof graphql_1.GraphQLInterfaceType) {
                                if (targetType.astNode) {
                                    hint = mergeHints(hint, cacheHintFromDirectives(targetType.astNode.directives));
                                }
                            }
                            const fieldDef = info.parentType.getFields()[info.fieldName];
                            if (fieldDef.astNode) {
                                hint = mergeHints(hint, cacheHintFromDirectives(fieldDef.astNode.directives));
                            }
                            if ((targetType instanceof graphql_1.GraphQLObjectType ||
                                targetType instanceof graphql_1.GraphQLInterfaceType ||
                                !info.path.prev) &&
                                hint.maxAge === undefined) {
                                hint.maxAge = defaultMaxAge;
                            }
                            info.cacheControl = {
                                setCacheHint: (dynamicHint) => {
                                    hint = mergeHints(hint, dynamicHint);
                                },
                                cacheHint: hint,
                            };
                            return () => {
                                if (hint.maxAge !== undefined || hint.scope !== undefined) {
                                    if (__testing__cacheHints) {
                                        const path = graphql_1.responsePathAsArray(info.path).join('.');
                                        if (__testing__cacheHints.has(path)) {
                                            throw Error("shouldn't happen: addHint should only be called once per path");
                                        }
                                        __testing__cacheHints.set(path, hint);
                                    }
                                    policyUpdater === null || policyUpdater === void 0 ? void 0 : policyUpdater.addHint(hint);
                                }
                            };
                        },
                    };
                },
                willSendResponse(requestContext) {
                    const { response, overallCachePolicy } = requestContext;
                    if (calculateHttpHeaders &&
                        overallCachePolicy &&
                        !response.errors &&
                        response.http) {
                        response.http.headers.set('Cache-Control', `max-age=${overallCachePolicy.maxAge}, ${overallCachePolicy.scope.toLowerCase()}`);
                    }
                },
            };
        },
    };
}
exports.ApolloServerPluginCacheControl = ApolloServerPluginCacheControl;
function cacheHintFromDirectives(directives) {
    if (!directives)
        return undefined;
    const cacheControlDirective = directives.find((directive) => directive.name.value === 'cacheControl');
    if (!cacheControlDirective)
        return undefined;
    if (!cacheControlDirective.arguments)
        return undefined;
    const maxAgeArgument = cacheControlDirective.arguments.find((argument) => argument.name.value === 'maxAge');
    const scopeArgument = cacheControlDirective.arguments.find((argument) => argument.name.value === 'scope');
    return {
        maxAge: maxAgeArgument &&
            maxAgeArgument.value &&
            maxAgeArgument.value.kind === 'IntValue'
            ? parseInt(maxAgeArgument.value.value)
            : undefined,
        scope: scopeArgument &&
            scopeArgument.value &&
            scopeArgument.value.kind === 'EnumValue'
            ? scopeArgument.value.value
            : undefined,
    };
}
function mergeHints(hint, otherHint) {
    if (!otherHint)
        return hint;
    return {
        maxAge: otherHint.maxAge !== undefined ? otherHint.maxAge : hint.maxAge,
        scope: otherHint.scope || hint.scope,
    };
}
function ApolloServerPluginCacheControlDisabled() {
    return {
        __internal_plugin_id__() {
            return 'CacheControl';
        },
    };
}
exports.ApolloServerPluginCacheControlDisabled = ApolloServerPluginCacheControlDisabled;
//# sourceMappingURL=index.js.map